#!/usr/bin/python
# ZPhotoSlides, Zope product Copyright (C) 2002-2003  Jean-Francois Roche <jfroche@jfroche.be> Florent Carlier
# <florent.carlier@insa-rennes.fr>
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# $Id: CMFPhoto.py,v 2.6 2004/08/31 20:29:31 jfroche Exp $'


from ZPhoto import Photo
from ZPhotoSlidesUtils import *
import AccessControl
from permissions import cmfphoto_apply_security, usePostcard, sendPostcard, exportPhoto
from Globals import InitializeClass
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from ZPhotoSlidesBase import ZPhotoSlidesBaseItem
from postalcard import PostalCards

factory_type_information = {
  'id'             : 'ZPhoto',
  'portal_type'    : 'CMF ZPhoto',
  'meta_type'      : 'CMF ZPhoto',                         # This is NOT a ZPhotoSlides
  'description'    : 'A photo with thumbnails.',
  'content_icon'   : 'photo.gif',                                # Icon has to be acquired from a skin folder
  'product'        : 'ZPhotoSlides',
  'factory'        : 'addZPhoto',
  'immediate_view' : 'zphoto_view',
  'global_allow'   : 0,
  'filter_content_types' : 1,                                    # This is for folderish types and zps is NOT folderish
  'allowed_content_types': (),
  'actions': (
    {
    'id'           : 'folder_content',
    'name'         : 'Contents',
    'action'       : 'python:object.aq_parent.absolute_url()+\'/zphotoslides_contents\'',
    'permissions'  : (ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'album',
    'name'         : 'album',
    'action'       : 'python:object.aq_parent.absolute_url()',
    'permissions'  : (View, ),
    'category'      : 'object',
    },
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'zphoto_view',
    'permissions'  : (View, ),
    'category'      : 'object',
    },
    {
    'id'           : 'postcard',
    'name'         : 'Send postcard',
    'action'       : 'postcard',
    'permissions'   : (sendPostcard, ),
    'category'      : 'object',
    },
    {
    'id'           : 'edit',
    'name'         : 'Edit',
    'action'       : 'image_edit_form',
    'permissions'   : (ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'transform',
    'name'         : 'Transform',
    'action'       : 'zphoto_edit_form',
    'permissions'   : (ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'           : 'export',
    'name'         : 'Export photo',
    'action'       : 'zphoto_export',
    'permissions'   : (exportPhoto, ),
    'category'      : 'object',
    },
    {
    'id'            : 'metadata',
    'name'          : 'Metadata',
    'action'        : 'metadata_edit_form',
    'permissions'   : (ModifyPortalContent, ),
    'category'      : 'object',
    },
    )
}


def addZPhoto(self,id,title='',file='',path=''):
    ''' Add a CMF ZPhoto '''
    img = ZPhoto(id,title,file)
    self._setObject(id,img)
    zp = self._getOb(id)
    if(have_IM != ''):
        zp.convert = have_IM
    #zp.init_position()
    if path:
        zp.update_data_from_fs(path)
        
#class CMFPhoto(Photo):
#    ''' CMF ZPhotoSlides '''


from Products.CMFDefault.Image import Image as CMFImage
class ZPhoto(Photo,ZPhotoSlidesBaseItem,PostalCards,CMFImage):
    ''' CMF ZPhoto '''

    __implements__ = ( CMFImage.__implements__ ,)

    security = AccessControl.ClassSecurityInfo()
    cmfphoto_apply_security(security,View,ModifyPortalContent)
    
    meta_type = 'CMF ZPhoto'
    
    manage_options = ( 
	               {'label': 'view',       'action': 'zphoto_view'},
                     )
    have_stripogram = have_stripogram
    have_localizer = have_localizer
    have_PIL = have_PIL
    have_IM = have_IM
    tmp_folder = tmp_folder

    def __init__(self,id,title='',file=''):
        try:
            Photo.__init__(self)
            ZPhotoSlidesBaseItem.__init__(self)
            PostalCards.__init__(self)
            CMFImage.__init__(self,id,title,file)
        except:
            Photo.__dict__['__init__'](self) #__init__(self)
            ZPhotoSlidesBaseItem.__dict__['__init__'](self) #__init__(self)
            PostalCards.__dict__['__init__'](self) #__init__(self)
            # not sure this works.. not yet tested with python 2.1
            CMFImage.__dict__['__init__'](self,id,title,file) #__init__(self,id,title,file)
        self._thumbs = {}

    def SearchableText(self):
        ''' texte to index in catalog '''
        return '%s %s' % ( self.Title(), self.Description(), )

    def countComments(self):
        dt=getToolByName(self, 'portal_discussion')
        if(dt.isDiscussionAllowedFor(self)):
            return len(dt.getDiscussionFor(self).objectValues())
        return None
    def showComments(self):
        dt=getToolByName(self, 'portal_discussion')
        return dt.isDiscussionAllowedFor(self)

InitializeClass(ZPhoto)
