#!/usr/bin/python
# ZPhotoSlides, Zope product Copyright (C) 2002-2003  Jean-Francois Roche <jfroche@jfroche.be> Florent Carlier
# <florent.carlier@insa-rennes.fr>
#
# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#


from ZPhotoSlidesFolder import PhotoSlidesFolder, manage_addZPhotoSlidesFolder
from ZPhotoSlidesBase import ZPhotoSlidesBaseItem
from permissions import cmfphotoslidesfolder_apply_security
from Globals import InitializeClass
import AccessControl
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.CMFDefault.SkinnedFolder import SkinnedFolder
from Products.CMFCore import CMFCorePermissions

factory_type_information = {
  'id'             : 'ZPhotoSlidesFolder',
  'portal_type'    : 'CMF ZPhotoSlides Folder',
  'meta_type'      : 'CMF ZPhotoSlides Folder',                         # This is NOT a ZPhotoSlides
  'description'    : 'A photo album folder',
  'content_icon'   : 'photofolder.gif',
  'product'        : 'ZPhotoSlides',                             # DON'T CHANGE THIS ! It MUST be ZPhotoSlides
  'factory'        : 'addZPhotoSlidesFolder',
  'immediate_view' : 'zphotoslidesfolder_view',
  'filter_content_types' : 1,                                    # This is for folderish types and zps is NOT folderish
  'allowed_content_types': ('ZPhotoSlides','ZPhotoSlidesFolder',),                    # We want only ZPhotoSlides content in ZPhotoSlidesFolder
  'actions': (
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'zphotoslidesfolder_view',
    'permissions'  : (View, ),
    'category'      : 'object',
    },
    #{
    #'id'           : 'folder_contents',
    #'name'         : 'Contents',
    #'action'       : 'folder_contents',                        # Why not using the Plone template ? ;)
    #'permissions'   : (ModifyPortalContent, ),
    #'category'      : 'object',
    #},
    {
    'id'           : 'edit',                     
    'name'         : 'Edit',
    'action'       : 'folder_edit_form',        # use plone template. It's quite clean
    'permissions'   : (ModifyPortalContent, ),
    'category'      : 'object',
    },
    # We dont need this. It's only use for comments/breadcrumbs.
    #{
    #'id'           : 'zphotoslidesfolder_edit_form',
    #'name'         : 'Properties',
    #'action'       : 'zphotoslidesfolder_edit_form',  # Our edit properties form without metadatas 
    #'permissions'   : (ModifyPortalContent, ),
    #'category'      : 'object',
    #},
    #{
    #'id'           : 'search',
    #'name'         : 'search',
    #'action'       : 'search_form',                        
    #'permissions'   : (ModifyPortalContent, ),                        
    #'category'      : 'object',
    #},
    {
    'id'            : 'metadata',
    'name'          : 'Metadata',
    'action'        : 'metadata_edit_form',
    'permissions'   : ('Manage properties', ),
    'category'      : 'object',
    },
    {
    'id'            : 'local_roles',
    'name'          : 'Sharing',
    'action'        : 'string:${folder_url}/folder_localrole_form',
    'permissions'   : ('Manage properties', ),
    'category'      : 'object',
    },
    {
    'id'            : 'folderlisting',
    'name'          : 'View',
    'action'        : 'zphotoslidesfolder_view',
    'permissions'   : (View, ),
    'category'      : 'folder',
    'visible'       : 0,
    },
    )
}


def addZPhotoSlidesFolder(self,id,title='',REQUEST=None):
    ''' add a CMF ZPhotoSlidesFolder '''
    position = len(self.objectValues(['CMF ZPhotoSlides Folder','CMF ZPhotoSlides',])) + 1
    zpsf = ZPhotoSlidesFolder(id,title)
    self._setObject(id,zpsf)
    zpsf = self._getOb(id)
    zpsf.manage_afterAdded('',position)
    if REQUEST:
        message = 'ZPhotoSlidesFolder added !'
        REQUEST.RESPONSE.redirect('%s/zphotoslidesfolder_edit?portal_status_message=%s' % (zpsf.absolute_url(),message))
    
class CMFPhotoSlidesFolder(PhotoSlidesFolder,ZPhotoSlidesBaseItem): # Use SkinnedFolder instead of Folder. It's a CMF content
    ''' CMF ZPhotoSlides '''
    
    meta_type = 'CMF ZPhotoSlides Folder'
    
    security = AccessControl.ClassSecurityInfo()
    cmfphotoslidesfolder_apply_security(security,View,ModifyPortalContent)
    
    manage_options = ( # what management options are there?
	               {'label': 'Contents',       'action': 'manage_main'},
	               {'label': 'edit',       'action': 'zphotoslides_edit_form'},
                     )
    #def folder_listing(self,REQUEST=None):
    #    ''' default view for folder '''
    #    if REQUEST:
    #        REQUEST.RESPONSE.redirect( '%s/zphotoslidesfolder_view' % self.absolute_url() )

    def SearchableText(self):
        ''' texte to index in catalog '''
        return '%s %s' % ( self.Title(), self.Description(), )
    
try:
    from Products.CMFPlone.PloneFolder import PloneFolder
    class ZPhotoSlidesFolder(CMFPhotoSlidesFolder,PloneFolder):
        ''' CMF ZPhotoSlides Folder '''

        security = AccessControl.ClassSecurityInfo()
        cmfphotoslidesfolder_apply_security(security,View,ModifyPortalContent)
        
        manage_options = PloneFolder.manage_options
        
        def __init__(self,id,title='',description=''):
            try:
                PloneFolder.__init__(self,id=id,title=title)
                ZPhotoSlidesBaseItem.__init__(self)
            except:
                PloneFolder.__dict__['__init__'](self,id=id,title=title) #__init__(self,id=id,title=title)
                ZPhotoSlidesBaseItem.__dict__['__init__'](self) #__init__(self)
            
    
except:
    class ZPhotoSlidesFolder(CMFPhotoSlidesFolder,SkinnedFolder):
        ''' CMF ZPhotoSlides Folder '''

        security = AccessControl.ClassSecurityInfo()
        cmfphotoslidesfolder_apply_security(security,View,ModifyPortalContent)
    
        manage_options = SkinnedFolder.manage_options
        
        def __init__(self,id,title='',description=''):
            try:
                SkinnedFolder.__init__(self,id=id,title=title,description=description)
                ZPhotoSlidesBaseItem.__init__(self)
            except:
                SkinnedFolder.__dict__['__init__'](self,id=id,title=title,description=description) #__init__(self,id=id,title=title,description=description)
                ZPhotoSlidesBaseItem.__dict__['__init__'](self) #__init__(self)

InitializeClass(ZPhotoSlidesFolder)
