from Globals import DTMLFile
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from OFS.Image  import cookId
from ZPhotoSlidesUtils import *
from Products.ZPhotoSlides.rating import RateItem
from Products.ZPhotoSlides.comment import CommentItem
from config import *

import os,time,re

class OrdererItem:
    ''' An Orderer Element '''
    
    def init_position(self):
        self._setProperty('position',len(self.aq_parent.objectValues(self.meta_type)),'int')
    
    def _getOrdererContent(self):
        if self.portal_type:
            return self.aq_parent.aq_explicit.listFolderContents()
            pass
        else:
            orderer = [(o.position,o) for o in self.aq_parent.objectValues(self.meta_type) if o.getviewit != '0']
            orderer.sort()
            return [o for (p,o) in orderer]
        
    def _move(self,position):
        if hasattr(self,'position'):
            newpos = self.position + position
            oldpos = self.position
            self.position = newpos
            for elem in self.aq_parent.objectValues([self.meta_type]):
                if elem.position == newpos and elem.getId() != self.getId():
                    elem.position = oldpos
                    
    def _getPos(self):
        objects = self._getOrdererContent()
        l = len(objects)
        for i in range( l ):
            if objects[i].getId() == self.getId():
                return i
        
    def getNextItem(self):
        '''Return the next object'''
        objects = self._getOrdererContent()
        l = len(objects)
        i = self._getPos()
        if i+1 >= l:
            return None
        else:
            return objects[i+1]

    def getPrevItem(self):
        '''Return the previous object'''
        objects = self._getOrdererContent()
        l = len(objects)
        i = self._getPos()
        if i == 0:
            return None
        else:
            return objects[i-1]
        
    def _getNavLink(self,pos,obj):
        if pos == self._getPos()+1:
            return '%i' % pos
        else:
            return '<a href="%s/view" title="%s">%s</a>' % (obj.absolute_url(),obj.title_or_id(),pos)

    def getNav(self):
        ''' return batch nav. A list of batch item '''
        nav = []
        objects = self._getOrdererContent()
        l = len(objects)
        size = 5                            # we show ( (size * 2) + 1 ) items in page list
        if not l or l <= 1:
            return None
        p = self._getPos()
        s_start = s_stop = 0
        if p > size:
            start = p - size
            if start > 1: s_start = 1
        else:
            start = 1
        if p < l - size:
            stop = p + size + 1
            if stop < l - 1: s_stop = 1
        else:
            stop = l - 1
        nav.append( self._getNavLink( 1, objects[0] ) )
        if s_start:
            nav.append('...')
        for i in range(start,stop):
                nav.append( self._getNavLink( i+1, objects[i] ) )
        if s_stop:
            nav.append('...')
        nav.append( self._getNavLink( l, objects[l-1] ) )
        return nav
                
    def moveUp(self,b_start=0,b_size=0,REQUEST=None):
        ''' move up one position '''
        items = self.aq_parent.objectValues([self.meta_type])
        if self.position != len(items):
            self._move(1)
        if(REQUEST):
            query = 'zps_message=%s&b_start=%i&b_size=%i' % ('Object moved up.',int(b_start),int(b_size))
            if self.meta_type in ('ZPhotoslides',):
                template = '%s/zphotoslides_contents?%s' % (self.absolute_url(),query)
            else:
                template = '%s/manage_position?%s' % (self.absolute_url(),query)
            return REQUEST.RESPONSE.redirect(template)

    def moveDown(self,b_start=0,b_size=0,REQUEST=None):
        ''' move down one position '''
        if self.position > 1:
            self._move(-1)
        if(REQUEST):
            query = 'zps_message=%s&b_start=%i&b_size=%i' % ('Object moved down.',int(b_start),int(b_size))
            if self.meta_type in ('ZPhotoslides',):
                template = '%s/zphotoslides_contents?%s' % (self.absolute_url(),query)
            else:
                template = '%s/manage_position?%s' % (self.absolute_url(),query)
            return REQUEST.RESPONSE.redirect(template)
        
class SkinnedItem:
    ''' A skinned item is an objet with template and css layout '''

    main = PageTemplateFile('zpt/main',globals())
    header = PageTemplateFile('zpt/header',globals())
    manage_skin = PageTemplateFile('zpt/manage_skin',globals())

    def __init__(self):
        self.text_color = ''
        self.link_color = self.link_hover = ''
        self.odd_color = self.odd_bg_color = ''
        self.even_color = self.even_bg_color = ''
        self.manage_changeColors()

    def manage_editMainTemplate(self,REQUEST=None):
        ''' redirect to main_template manage page '''
        if REQUEST:
            REQUEST.RESPONSE.redirect('%s/main_template/manage_main' % self.absolute_url() )
        
    def manage_editCSS(self,REQUEST=None):
        ''' redirect to style sheet manage page '''
        if REQUEST:
            REQUEST.RESPONSE.redirect('%s/zps_style.css/manage_main' % self.absolute_url() )
            
    def manage_delSkin(self,ids=['main_template','zps_style.css'],REQUEST=None):
        ''' delete  Skin object (main_template/style sheet) '''
        for id in ids:
            try: self.manage_delObjects(ids=[id])
            except: pass
        if REQUEST:
            message = '%s have been delete.' % ' '.join(ids)
            REQUEST.RESPONSE.redirect('%s/manage_skin?zps_message=%s' % ( self.absolute_url(), message ))
        
    def isSkinned(self):
        ''' return 1 if object have a main_template and style sheet '''
        if 'main_template' in self.objectIds():
            return 1
        return 0

    def manage_restoreSkin(self,REQUEST=None):
        ''' restore the default zps skin '''
        self.manage_restoreZpt()
        self.manage_restoreCSS()
        if REQUEST:
            message = 'Your skin have been restore'
            REQUEST.RESPONSE.redirect('%s/manage_skin?zps_message=%s' % ( self.absolute_url(), message ))
            
    def manage_restoreZpt(self,REQUEST=None):
        ''' restore the default template '''
        main_template = PageTemplateFile('zpt/main_template',globals())
        filename = main_template.filename
        f = open(filename,"rb")
        text = ''
        text += f.read()
        self.manage_delSkin(['main_template'])
        self.manage_addProduct['PageTemplates'].manage_addPageTemplate(id='main_template', title='', text=text)

        if REQUEST:
            message = 'Your main_template have been restore'
            REQUEST.RESPONSE.redirect('%s/manage_skin?zps_message=%s' % ( self.absolute_url(), message ))

    def manage_restoreCSS(self,REQUEST=None):
        ''' restore the default style sheet '''
        main_template = PageTemplateFile('zpt/main_template',globals())
        filename = main_template.filename
        filename = main_template.filename.replace('main_template.zpt','style_css.dtml')
        f = open(filename,"rb")
        text = '<dtml-let text_color="\'%s\'" link_color="\'%s\'" link_hover="\'%s\'" odd_color="\'%s\'" odd_bg_color="\'%s\'" even_color="\'%s\'" even_bg_color="\'%s\'">\n' % (
                self.text_color,
                self.link_color, self.link_hover,
                self.odd_color, self.odd_bg_color,
                self.even_color, self.even_bg_color,
            )
        text += f.read()
        text += '\n</dtml-let>'
        self.manage_delSkin(['zps_style.css'])
        self.manage_addDTMLMethod('zps_style.css','',text)

        if REQUEST:
            message = 'Your CSS have been restore'
            REQUEST.RESPONSE.redirect('%s/manage_skin?zps_message=%s' % ( self.absolute_url(), message ))

    def manage_restoreColors(self,REQUEST=None):
        ''' restore the default colors '''
        self.manage_changeColors() 
        if REQUEST:
            message = 'Colors have been changed.'
            REQUEST.RESPONSE.redirect('%s/manage_skin?zps_message=%s' % ( self.absolute_url(), message ))

    def manage_changeColors(self,
                     text_color='#000000',
                     link_color='#000080', link_hover='#005050',
                     odd_color='#003333', odd_bg_color='#dfdfdf',
                     even_color='#dfdfdf', even_bg_color='#999999',
                     REQUEST=None
                    ):
        ''' change colors '''
        self.text_color = text_color
        self.link_color = link_color
        self.link_hover = link_hover
        self.odd_color = odd_color
        self.odd_bg_color = odd_bg_color
        self.even_color = even_color
        self.even_bg_color = even_bg_color
        self.manage_restoreCSS() 
        if REQUEST:
            message = 'Colors have been changed.'
            REQUEST.RESPONSE.redirect('%s/manage_skin?zps_message=%s' % ( self.absolute_url(), message ))
           
class UtilsItem:
    ''' Utils Item 
    Authorized ZPhotoSlidesUtils methods'''
    
    def getConfig(self):
        ''' return zps config '''
        return zpsconfig()

    def getTmpFolder(self):
        ''' return tmp folder '''
        return tmp_folder

    def findPil(self):
        ''' return True if we have PIL lib '''
        return have_PIL

    def findConvert(self):
        ''' return true if we have IM convert binary '''
        return have_IM

class ZPhotoSlidesBaseItem(OrdererItem,RateItem,UtilsItem):
    ''' Base Orderer/Rated item '''

    
    def __init__(self):
        try:
            RateItem.__init__(self)
        except:
            RateItem.__dict__['__init__'](self) #__init__(self)
        self.show_comment = 1
        self.version = VERSION

    def needupgrade(self,REQUEST):
        ''' return true if we need upgrade '''
        if self.version != VERSION:
            return 1
        return 0
        
    def upgrade(self,REQUEST):
        ''' Allow the user to Upgrade to the object '''
        # nothing now
        pass

    def showComments(self):
        ''' return 1 if object use comment 
        This is for template compatibility '''
        return 0

    def number_of_comments(self,*args):
        ''' fake '''
        return 0
    
    
    def getManagerMail(self):
        ''' return manager mail '''
        manager_mail = self._ac_attr('manager_mail')
        if not manager_mail:
            manager_mail = 'postcard-no-spam@no-spam-zphotoslide.org'
        return manager_mail

    def _ac_attr(self,attr):
        # acquire attr if needed
        if self.isPrincipiaFolderish:
            return getattr(self,attr)
        elif hasattr(self.aq_parent,attr) and self.aq_parent.isPrincipiaFolderish:
            return getattr(self.aq_parent,attr)
        elif hasattr(self,attr):
            return getattr(self,attr)
        return None

    def getFormats(self,only_used=0):
        ''' return a sorted list of dicts {name,size,used} of the availables formats '''
        result = []
        formats_items = self._ac_attr('formats_available').values()
        if only_used:
            formats = [(i['size'],i) for i in formats_items if i['used'] == 1]
        else:
            formats = [(i['size'],i) for i in formats_items]
        formats.sort()
        return [dict for (size,dict) in formats]
    
    def getFormat(self,size='thumb'):
        ''' return a sorted list of dicts {name,size,used} of the availables formats '''
        formats = self.getFormats()
        for format in formats:
            if format['name'] == size:
                return format

        # this sucks. Don't know why...
        #formats = self._ac_attr('formats_available')
        #if formats.has_key(size): return formats[size]
        return None

    def getSizes(self,only_used=1):
        formats = self.getFormats(only_used=only_used)
        return [f['name'] for f in formats]

    def getDefaultSize(self):
        ''' return default image size '''
        return self._ac_attr('default_photo_size')

    def showPostcard(self):
        ''' return 1 if object use postcard '''
        return self._ac_attr('show_postcard')
        
    def showARPostcard(self):
        ''' return 1 if object use postcard '''
        return self._ac_attr('show_ARpostcard')
    
    def validEmail(self,address):
	"Return true if the supplied email address is in a valid format"
        if address:
	    if re.search(r"^([\w_\.\-])+\@(([\w\-])+\.)+([\w]{2,4})+$", address):
	       return 1
        return 0

    def external_edit(self):
        ''' Redirect while zps does not support external edit '''
        return """
                <html>
                <head>
                <title>External Edit</title>
                </head>
                <body>
                <center>
                <h4>External Editor is not yet supported... Sorry.</h4>
                <input type="button" value="Back" onclick="javascript:history.go(-1)"/>
                </center>
                </body>
                </html>
                """

    def _createDirectory(self,path=''):
        # test if we have write access to path and create the zps_thumb dir if needed
        if path:
            path = str(path)
            if os.access(path,os.R_OK):
                thumbsdir = os.path.join(path,'zps_thumbs')
                if(not os.path.isdir(thumbsdir)): os.mkdir(thumbsdir)
                return thumbsdir
            else:
                raise IOError,'Cant write to %s. Verify your permissions setting.' % path

class ZPhotoSlidesBaseItemWithComment(CommentItem,ZPhotoSlidesBaseItem):
    ''' Base Orderer/Rated item with Comment '''

    zps_colophon = PageTemplateFile('skins/zphotoslides/zps_colophon',globals())

    def __init__(self):
        try:
            ZPhotoSlidesBaseItem.__init__(self)
            CommentItem.__init__(self)
        except:
            ZPhotoSlidesBaseItem.__dict__['__init__'](self) #__init__(self)
            CommentItem.__dict__['__init__'](self) #__init__(self)
        self.show_breadcrumbs = 0

    def showBreadcrumbs(self):
        ''' return true if we have to show breadcrumbs '''
        return self._ac_attr('show_breadcrumbs')
    
    def getBreadcrumbs(self, include_root=1,REQUEST=None):
        ''' return breadcrumbs '''
        if not REQUEST: REQUEST = self.REQUEST
        if REQUEST:
            crumb = []
            site_path = REQUEST.get('BASE0',None)
            if site_path:
                path = self.absolute_url().replace(site_path,'')
                liste = path.split('/')[1:]
                liste.reverse()
                current = self
                for id in liste:
                    if current.getId() == id:
                        crumb.append( '<a href="%s">%s</a>' % ( current.absolute_url(), current.title_or_id() ) )
                        current = current.aq_parent
                crumb.reverse()
                return ' / '.join(crumb)
        return ''
