# ZPhotoSlides, Zope product
# Copyright (C) 2002-2003  Jean-Francois Roche <jfroche@jfroche.be>
#                          Florent Carlier <florent.carlier@insa-rennes.fr>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# $Id: ZPhotoSlidesFolder.py,v 2.7 2004/10/09 16:31:33 gawel Exp $' 

"""ZPhotoSlides Folder
"""

from Globals import Persistent
from Globals import package_home
from OFS.ObjectManager import ObjectManager
from OFS.SimpleItem import Item
from OFS.PropertyManager import PropertyManager
from OFS.FindSupport import FindSupport
from OFS.Folder import Folder
from AccessControl.Role import RoleManager
import AccessControl
from Acquisition import aq_base
from Globals import MessageDialog
from Globals import InitializeClass, HTMLFile, HTML
from ZPhotoSlides import ZPhotoSlides
from ZPhotoSlidesBase import ZPhotoSlidesBaseItemWithComment,SkinnedItem
from permissions import zphotoslidesfolder_apply_security
from Globals import DTMLFile
from Products.ZCatalog.ZCatalog import ZCatalog
import os
import string
from DateTime import DateTime
from ZPhotoSlidesUtils import *
from ZPSPloneBatch import Batch
import whrandom
import time
from stat import stat



if have_localizer:
	from Products.Localizer import LocalDTMLFile
	from Products.Localizer import LocalPageTemplateFile
	from Products.Localizer import Gettext
	from Products.Localizer import LanguageManager
	zpsgettext = Gettext.translation(globals())
else:
	from Globals import DTMLFile
	from Products.PageTemplates.PageTemplateFile import PageTemplateFile
	LocalPageTemplateFile = PageTemplateFile
	LocalDTMLFile = DTMLFile
	del PageTemplateFile
	del DTMLFile

if(not have_localizer):
	from ZPhotoSlidesUtils import zpsgettext
_ = zpsgettext

class PhotoSlidesFolder:
    """ZPhotoSlides Folder object implementation."""
    
    _properties = (
        {'id':'title', 'type': 'string', 'mode': 'w'},
        )



    def manage_afterAdded(self,description,position):
	self.position = position
	self.show_breadcrumbs = 1
        self.update_date = '12/03/2004'
        self.zphoto_header_folder= ''                  # get content of standard header Folder
        self.zphoto_footer_folder= ''                  # get content of standard header Folder
        self.keywords = ''                      # keywords describing the object
        self.description = HTML(description)                   # description of the object
	self.last_modif = DateTime()            # Why don't use the bobo_base_modification_time ? ;)
        if not self.portal_type:
            if not hasattr(self,'main_template'):
                self.manage_restoreSkin()
	    catalog = ZCatalog('catalog',container=self)
	    self._setObject('catalog', catalog)
	    self.manage_catalog()


    def getBatch(self, files, b_size, b_start, orphan):
      """ Patch ZPT """
      return Batch(files, b_size, int(b_start), orphan=0)

    def getZPhoto(self,suppressHiddenFiles=0,recursive=0):
        ''' Return the list of ZPhoto objects '''
        list = []
        for zps in self.getZPhotoSlides(recursive=recursive):
            for zph in zps.imagesValues():
                list.append(zph)
        return list

    def getAllSortedZPhoto(self,size):
        items = self.getZPhoto(recursive=1)
        items.sort(lambda x,y: -cmp(x.visits_log.total_visits(),y.visits_log.total_visits()))
        return items[:size]

    def getZPhotoSlides(self,suppressHiddenFiles=0,recursive=0):
        ''' return ZPhotoSlides objects '''
        list = self.objectValues(zps_meta_types)
        if(recursive==1):
            for zps_folder in self.getZPhotoSlidesFolder():
                list.append(zps_folder.getZPhotoSlides())
        return list

    def getOrderedVisits(self):
        "Return the list of most visited zphotoslides in this folder"
        items = self.getZPhotoSlides(recursive=1)
        items.sort(lambda x,y: -cmp(x.visits_log.total_visits(),y.visits_log.total_visits()))
        return items

    def getZPhotoSlidesFolder(self,suppressHiddenFiles=0):
        ''' return ZPhotoSlides Folders objects '''
        return self.objectValues(zpsfolder_meta_types)
    
    def getZPhotoSlidesFolderish(self,suppressHiddenFiles=0):
        ''' return ZPhotoSlides Folderich objects '''
        if self.portal_type:
            return self.aq_explicit.listFolderContents()
        else:
            return self.objectValues(zpsfolderish_meta_types)
    
    def manage_edit(self, title,description,show_breadcrumbs=0,REQUEST=None,id=''):
        """Edit the main properties..."""
        if id: self.id = id
        if title: self.title = title
	if(description!=''):
	      self.description = HTML(description)
	else:
	      self.description = ''
        for item in ('show_breadcrumbs',):
            if eval(item) in ('on',1):
                setattr(self,item,1)
            else:
                setattr(self,item,0)
        if not self.portal_type and REQUEST:
            self.manage_edit_comments(REQUEST)
        # what about bobobase_modification_time ?
        self.last_modif = DateTime()
        message = "Properties for %s changed." % self.id
        if REQUEST is not None:
            return REQUEST.RESPONSE.redirect( '%s/zphotoslidesfolder_edit_form?zps_message=%s' %
                (self.absolute_url(),message))


    def zphotoslidesIds(self,suppressHiddenFiles=0):
        """Return list of ZPhotoSlides in this folder."""
        return [zps.getId() for zps in self.getZPhotoSlides(suppressHiddenFiles)] 

    def getAllPhotos(self,suppressHiddenFiles=0):
	list = []
        for zphoto in self.getZPhotoSlides(suppressHiddenFiles): 
		list = list + zphoto.imagesValues(suppressHiddenFiles)
        for folder in self.getZPhotoSlidesFolder(suppressHiddenFiles):
		list = list + folder.getAllPhotos(suppressHiddenFiles)
	return list

    def numZPhotoSlides(self,suppressHiddenFiles=0):
        """Return number of ZPhotoSlides in folder tree."""
        ZPhotoSlides = len(self.getZPhotoSlides()) 
        for folder in self.getZPhotoSlidesFolder():
            ZPhotoSlides = ZPhotoSlides + folder.numZPhotoSlides()
        return ZPhotoSlides

    def numZPhotoSlidesFolder(self,suppressHiddenFiles=0):
	nbr = len(self.getZPhotoSlidesFolder(suppressHiddenFiles)) 
        for folder in self.getZPhotoSlidesFolder(suppressHiddenFiles):
            nbr = nbr + folder.numZPhotoSlidesFolder(suppressHiddenFiles)
	return nbr

    def get_thumb_random(self,thumb_album):
        photorandom = whrandom.choice(thumb_album.imagesValues(1))
	return photorandom

    def get_nbr_non_hidden_thumb(self,suppressHiddenFiles=0):
	"return the number of photos inside all the ZPhotoSlides of the folder"
	sum = 0
        for zphoto in self.getZPhotoSlides(suppressHiddenFiles):
		sum = sum + len(zphoto.imagesValues(1)) 
        for album in self.getZPhotoSlidesFolder(suppressHiddenFiles):
		sum = sum + album.get_nbr_non_hidden_thumb()
	return sum

    def give_header_4_robots(self):
	"give the header for the robots"
	return """
<meta name="KEYWORDS" content="%s" />
<meta name="DESCRIPTION" content="%s" />
<meta name="resource-type" content="document" />
<meta name="ROBOTS" content="INDEX,FOLLOW" />
<meta http-equiv="expires" content="0" />
<meta name="author" content="ZPhotoSlides Folder" />
<meta name="copyright" content="ZPhotoSlides Folder %s" />
<meta name="revisit-after" content="1 days" />
<meta name="rating" content="General" />""" % (self.keywords,self.description,self.version)

    def __len__(self):
        l = self.getZPhotoSlidesFolderish()
        if l:
	    return len(l) #objectValues(['ZPhotoSlides','ZPhotoSlides Folder']))
        else:
            return 0

    def index(self,item):
	list = self.getZPhotoSlidesFolderish() #objectValues(['ZPhotoSlides','ZPhotoSlides Folder'])
	list.sort(lambda x,y: cmp(x.position,y.position))
	return list.index(item)

manage_addZPhotoSlidesFolderForm = LocalDTMLFile('dtml/addZPhotoSlidesFolderForm', globals(),zpsgettext=zpsgettext)

def get_folder_list(folder): # get the list of folders
    pass
    

def manage_addZPhotoSlidesFolder(dispatcher, id, title='',description='',folder='', REQUEST=None):
    """Add ZPhotoSlides Folder object"""
    dest = dispatcher.Destination()
    position = len(dispatcher.objectValues(zpsfolderish_meta_types)) +1 #['ZPhotoSlides Folder','ZPhotoSlides',])) + 1
    dest._setObject(id, ZPhotoSlidesFolder(id, title))
    self = dest._getOb(id)
    self.manage_afterAdded(description,position)
    if(folder!=''):
	if(not os.path.isdir(folder)):
		raise IOError,'Error: %s : folder unknown!!' % photo_folder
	else:
		pass	    

    if REQUEST is not None:
        message = 'ZPhotoSlideFolder has been created.'
        REQUEST.RESPONSE.redirect('%s/zphotoslidesfolder_edit_form?zps_message=%s' % (self.absolute_url(), message) )
    return id


class ZPhotoSlidesFolder(PhotoSlidesFolder,ZPhotoSlidesBaseItemWithComment,SkinnedItem,FindSupport,Folder):
    ''' ZPhotoSlides Folder object implementation. '''

    security = AccessControl.ClassSecurityInfo()
    zphotoslidesfolder_apply_security(security,'View','View management screens')
    

    meta_type = "ZPhotoSlides Folder"
    portal_type = None

    manage_options = (
                       {'label': "Content",    'action':'manage_main'}, # for developement
		       {'label': "View",	'action': 'index_html'},
	               {'label': "Properties",       'action': 'zphotoslidesfolder_edit_form'},
                       {'label': "Edit skin", 'action': 'manage_skin'},
		       {'label': "Manage positions",'action': 'manage_position'},
		       {'label': "Search for photos",'action': 'search_photo'},
		       {'label': "Statistics",'action': 'manage_stats'},
                       {'label': "Security",   'action': 'manage_access'},
                       {'label': "Undo", 'action': 'manage_UndoForm'},
                       {'label': "Ownership", 'action': 'manage_owner'},
		       {'label': "About this product", 'action':'about'}
		     )

    index_html = LocalPageTemplateFile('skins/zphotoslides/zphotoslidesfolder_view',globals())
    view = LocalPageTemplateFile('skins/zphotoslides/zphotoslidesfolder_view',globals())
    zphotoslidesfolder_view = LocalPageTemplateFile('skins/zphotoslides/zphotoslidesfolder_view',globals())
    zphotoslidesfolder_edit_form = LocalPageTemplateFile('skins/zphotoslides/zphotoslidesfolder_edit_form',globals())
    zphotoslides_header = LocalPageTemplateFile('skins/zphotoslides/zphotoslides_header',globals())
    zps_batch_macros = LocalPageTemplateFile('skins/zphotoslides/zps_batch_macros',globals(),gettext=zpsgettext)
    abouts = LocalPageTemplateFile('skins/zphotoslides/abouts',globals())
    search_photo = LocalDTMLFile('dtml/search_photo',globals(),zpsgettext=zpsgettext)
    zphotoslides_tree = LocalPageTemplateFile('skins/zphotoslides/zphotoslides_tree',globals())
    zps_tree = LocalDTMLFile('skins/zphotoslides/zps_tree',globals())
    manage_stats = LocalPageTemplateFile('skins/zphotoslides/zphotoslidesfolder_stats',globals())

    # old dtml
    manage_position = LocalDTMLFile('dtml/manage_ZPS_position',globals(),zpsgettext=zpsgettext)
    manage_tabs = LocalDTMLFile('dtml/manage_tabs',globals(),zpsgettext=zpsgettext)
    search_result = LocalDTMLFile('dtml/search_result',globals(),zpsgettext=zpsgettext)
    manage_main = LocalDTMLFile('dtml/list_zphotoslides',globals(),zpsgettext=zpsgettext)
    zps_content = LocalDTMLFile('dtml/list_zphotoslides',globals(),zpsgettext=zpsgettext)

    def __init__(self, id, title=''):
        try:
            ZPhotoSlidesBaseItemWithComment.__init__(self)
        except:
            ZPhotoSlidesBaseItemWithComment.__dict__['__init__'](self)
        SkinnedItem.__init__(self)
        Folder.__init__(self,id=id)
        if title:
            self.title = title

    def getIcon(self):
        ''' return icon location '''
        return '%s/misc_/ZPhotoSlides/photofolder.gif' % self.absolute_url()

    def manage_catalog(self):
	try:    #for 2.5 ZCatalog which add automaticaly id and title and bobobase_modification_time
		self.catalog.manage_delIndex(ids=['id','title','bobobase_modification_time'])
	except:
		pass
	self.catalog.addIndex('id','FieldIndex')
	self.catalog.addIndex('title','TextIndex')
	self.catalog.addIndex('keywords','TextIndex')
	try:
		self.catalog.addIndex('bobobase_modification_time','DateIndex')
	except:	#zope 2.5 has no DateIndex =(
		self.catalog.addIndex('bobobase_modification_time','FieldIndex')
 
    def reCatalog(self, REQUEST=None):
	"""clear the catalog, and put all images into it."""
        self.catalog._catalog.clear()
        path='/'+string.split(REQUEST.URL1, REQUEST.SERVER_URL)[1][1:]
	self.catalog.ZopeFindAndApply(self, obj_metatypes=['Image'], search_sub=1,REQUEST=REQUEST, apply_func=self.catalog.catalog_object, apply_path=path)
	return self.search_photo(manage_tabs_message="Objects recataloged!")


    # have to move this in ZPSBases.OrdererItem
    def change_order_by_date(self,REQUEST=None):
	"update position of the different zphotoslides and zps fallowing the date of modification"
	list_zp = self.getZPhotoSlidesFolderish() #objectValues(['ZPhotoSlides','ZPhotoSlides Folder'])
	list_zp.sort(lambda x, y: cmp(x.bobobase_modification_time,y.bobobase_modification_time))
	cpt = 1
	for item in list_zp:
		item.position = cpt
		cpt = cpt + 1
        if(REQUEST!=None):
	    return self.manage_position(manage_tabs_message=_("Positions updated!"))

    # have to move this in ZPSBases.OrdererItem
    def change_order(self,REQUEST):
	"change the order of the zphotoslides and zps"
        for zphoto_item in self.getZPhotoSlidesFolderish(): #objectValues(['ZPhotoSlides','ZPhotoSlides Folder']):
		item = zphoto_item.getId() + '_position'
		for key in REQUEST.form.keys():
			if(item == key):
				zphoto_item.position = int(REQUEST.form[key])
	return self.manage_position(manage_tabs_message="Positions updated!",zpsgettext=zpsgettext)



InitializeClass(ZPhotoSlidesFolder)
