# ZPhotoSlides, Zope product
# Copyright (C) 2002-2003  Jean-Francois Roche <jfroche@jfroche.be>
#                          Florent Carlier <florent.carlier@insa-rennes.fr>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# $Id: comment.py,v 2.1 2004/07/16 12:13:21 jfroche Exp $'

from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from OFS.SimpleItem import SimpleItem
from ZPhotoSlidesUtils import *

from stat import ip_log
import time,random

class comment(SimpleItem):
    """a comment"""
    def __init__(self,id,commt,user,place,date,ip,email):
        self.id = id
        self.ip = ip
        self.commt = commt
        self.date = date
        self.user = user
        self.place = place
        self.email = email
       
    def edit(self,id,commt,user,place,date,ip,email):
        self.ip = ip
        self.commt = commt
        self.date = date
        self.user = user
        self.place = place
        self.email = email

class CommentItem:
    ''' A commentable item '''

    zps_comments = PageTemplateFile('zpt/zps_comments',globals())
    zps_popup = PageTemplateFile('skins/zphotoslides/zps_popup',globals())
    
    def __init__(self):
        self.list_comment = {}
        self.show_comment = 1
        self.popup_comments = 1
        self.popup_add_comment = 1
    
    def manage_edit_comments(self,REQUEST=None):
        ''' manage edit for comments '''
        if REQUEST:
            show_comment = REQUEST.get('show_comment',None)
            popup_comments = REQUEST.get('popup_comments',None)
            popup_add_comment = REQUEST.get('popup_add_comment',None)
            if show_comment:
                self.show_comment = 1
            else: 
                self.show_comment = 0
            if popup_comments:
                self.popup_comments = 1
            else:
                self.popup_comments = 0
            if popup_add_comment:
                self.popup_add_comment = 1
            else:
                self.popup_add_comment = 0

    def getComments(self):
        ''' return all comments '''
        return self.list_comment.values()

    def addComment(self,name,commt,place,email,photo_id=None,REQUEST=None):
        ''' add comment '''
        #Unique random id
        id = time.strftime('%Y-%s-%d-%H-%M-%S', time.localtime()) + str(random.random())[4:]
        times = time.strftime('%H:%M:%S %d-%m-%Y', time.localtime())
	clean_commt = html2safehtml(commt, valid_tags=('b','a','i','br','p'))
#        iplog = stat().ip_log(REQUEST)
        iplog = ip_log(REQUEST)
        c = comment(id,clean_commt,name,place,times,iplog,email)
        self.list_comment[id] = c
        message = "Comment added"
        if REQUEST is not None:
            if self.popup_add_comment:
                return REQUEST.RESPONSE.redirect( '%s/zps_popup?zps_message=%s' %
                                                            (self.absolute_url(),message))
            else:
                return REQUEST.RESPONSE.redirect( '%s/view?zps_message=%s' %
                                                            (self.absolute_url(),message))

    def editComment(self,id,old_date,date,name,email,place,commt,REQUEST=None):
	"Manager modify a comment"
        c = self.list_comment[id]
	times = time.strftime('%H:%M:%S %d-%m-%Y', time.localtime()) # when the comment is add
	clean_commt = html2safehtml(commt, valid_tags=('b','a','i','br','p'))
        c.edit(id,clean_commt,name,place,times,iplog,email)
        message = 'Comment update'
        if REQUEST is not None:
            if self.popup_comments:
                return REQUEST.RESPONSE.redirect( '%s/zps_popup?macro=view_comments&zps_message=%s' %
                                                            (self.absolute_url(),message))
            else:
                return REQUEST.RESPONSE.redirect( '%s/view?zps_message=%s' %
                                                            (self.absolute_url(),message))

    def delComments(self,ids=[],REQUEST=None):
	'''Delete a comment from a photo fallowing the comment date'''
        for id in ids:
            if self.list_comment.has_key(id):
                del self.list_comment[id]
        message = 'Comments delete'
        if REQUEST is not None:
            if self.popup_comments:
                return REQUEST.RESPONSE.redirect( '%s/zps_popup?macro=view_comments&zps_message=%s' %
                                                            (self.absolute_url(),message))
            else:
                return REQUEST.RESPONSE.redirect( '%s/view?zps_message=%s' %
                                                            (self.absolute_url(),message))
        
    def hasComment(self,photo_id=None):
	'''Has a comment for the photo '''
        if self.show_comment:
            return self.countComments()
	return 0
        
    def showComments(self):
        ''' return 1 if object use comment '''
        return self._ac_attr('show_comment')

    def countComments(self):
        ''' nb of comment '''
        return len(self.list_comment.items())
