#!/usr/bin/python

# ZPhotoSlides, Zope product
# Copyright (C) 2002-2003  Jean-Francois Roche <jfroche@jfroche.be>
#                          Florent Carlier <florent.carlier@insa-rennes.fr>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# $Id: IMG_fileobj.py,v 2.1 2004/07/16 12:13:33 jfroche Exp $'

import os, sys, time

class FileObj:
	def __init__(self, filePath, col=50):
		self.id = ('img')
		self.filePath = os.path.abspath(os.path.normpath(filePath))
		
		self.pathExist = os.path.exists(self.filePath)
		
		if not self.pathExist:
			 sys.exit("Path %s does not exist"%filePath)

		self.fileName = os.path.basename(self.filePath)
		self.col = col
		
		if not self.fileName:
			self.fileName = os.sep
			
	def getParent(self):
		if os.path.dirname(self.filePath) == self.filePath:
			self.parentDir = None
		else:
			self.parentDir = os.path.dirname(self.filePath)
		return self.parentDir
		
	def isDir(self):# est-ce un repertoire ?
		if os.path.isdir(self.filePath):
			return 1
		else:
			return 0
	
	def isFile(self):# est-ce un fichier ?
		if os.path.isfile(self.filePath):
			return 1
		else:
			return 0
	
	def getDir(self):
		if self.isDir():
			fileList = []
			dirList = []
			fileList = os.listdir(self.filePath)
			for file in fileList:
				if os.path.isdir(os.path.join(self.filePath, file)):
					dirList.append(os.path.join(self.filePath, file))
			return dirList
		else:
			return None

	def getFiles(self):
		if self.isDir():
			fileList = []
			rawFileList = []
			rawFileList = os.listdir(self.filePath)
			for file in rawFileList:
				if os.path.isfile(os.path.join(self.filePath, file)):
					fileList.append(os.path.join(self.filePath, file))
			return fileList
		else:
			return None

	def visit(self, arg, dirName, files):
		self.recursiveDirList.append(dirName)
	
	def getRecursiveDirs(self):
		if self.isDir():
			self.recursiveDirList = []
			os.path.walk(self.filePath, self.visit, None)
			return self.recursiveDirList
		return None

	def getRecursiveFiles(self):
		if self.isDir():
			self.recursiveFileList = []
			dirList = self.getRecursiveDirs()
			for dir in dirList:
				files = os.listdir(dir)
				for file in files:
					if os.path.isfile(os.path.join(dir, file)):
						self.recursiveFileList.append(os.path.join(dir, file))
			return self.recursiveFileList
		return None

	def getStat(self):
		stat_list = []
		stat_list = os.stat(self.filePath)
		return stat_list
	
	def getLastAccess(self):
		a = self.getStat()
		return time.ctime(a[7])
	
	def getLastModif(self):
		a = self.getStat()
		return time.ctime(a[8])
	
	def getLastStatus(self):
		a = self.getStat()
		return time.ctime(a[9])
	
	def getSize(self):
		if self.isFile():
			a = self.getStat()
			return a[6]
		else:
			dirSize = 0
			for i in self.getBiggerFileList():
				dirSize = dirSize + i[1]
			return dirSize

	def cmpSize(self, first, second):
		if int(first[1]) < int(second[1]):
			return 1
		elif int(first[1]) == int(second[1]):
			return 0
		else:
			return -1

	def getBiggerFileList(self):
		biggerFileList = []
		for file in self.getFiles():
			a = FileObj(file)
			name_size_struct = (a.filePath, a.getSize())
			biggerFileList.append(name_size_struct)
		biggerFileList.sort(self.cmpSize)
		return biggerFileList

	def printBiggerFileList(self):
		maxi = []
		for struct in self.getBiggerFileList():
			maxi.append(len(struct[0]))
		self.col = max(maxi)
		for struct in self.getBiggerFileList():
			sl = len(struct[0])
			pad = self.col - sl
			print "%s:%s%s bytes" %(struct[0], " "*pad, struct[1]), self.col
				
	def __str__(self):
		filepath = self.filePath
		parentdir = self.getParent()
		filesize = self.getSize()
		isfile = self.isFile()
		isdir = self.isDir()
		access_modif_status = (self.getLastAccess(), self.getLastModif(), self.getLastStatus())
		currentdirlist = self.getDir()
		currentfiles = self.getFiles()
		alldirs = self.getRecursiveDirs()
		allfiles = self.getRecursiveFiles()
		fileIdentity = "filepath : %s\nparent dir: %s\nfile size : %s bytes\nfile? %s\ndir ? %s\n"%(filepath, parentdir, filesize, isfile, isdir)
		curdirliststring, curfileliststring, alldirstring, allfilestring = "", "", "", ""
		if self.isDir():
			for dir in currentdirlist:
				curdirliststring = curdirliststring + dir + "\n"
			for file in currentdirlist:
				curfileliststring = curfileliststring + file + "\n"
			for dir in alldirs:
				alldirstring = alldirstring + dir + "\n"
			for file in allfiles:
				allfilestring = allfilestring  + file + "\n"
		else:
			curdirliststring, curfileliststring, alldirstring, allfilestring = None, None, None, None
		mode = "last access : %s\nlast modification : %s\nlast status : %s\n"%(access_modif_status[0], access_modif_status[1], access_modif_status[2])
		listings = "dirs : %s\nfiles : %s\nall dirs : %s\nall files : %s\n"%(curdirliststring, curfileliststring, alldirstring, allfilestring)
		objectString = "%s%s%s"%(fileIdentity, mode, listings)
		return objectString
		
def main():
	try:
		path = sys.argv[1]
	except IndexError:
		path = os.curdir
	
	try:
		col = int(sys.argv[2])
	except IndexError:
		col = 50
	
	a = FileObj(path, col)
	
	a.printBiggerFileList()
	#print a
	
if __name__ == "__main__":
	main()
