import os
from os import X_OK
import sys
import string
import traceback
from cStringIO import StringIO 

class IM_Image_modif:
    def __init__(self, convert_location, data):
        if(not os.access(convert_location,X_OK)):
            raise IOError,"Can't execute Image Magick! Check location of Image magick, check that zope user has the right to run convert"
        else:
            self.convert = convert_location
        if sys.platform == 'win32':
            self.identify = os.path.join(os.path.dirname(self.convert),'identify.exe')
        else:
            self.identify = os.path.join(os.path.dirname(self.convert),'identify')
        self.data = data

    def filter_blur(self):
        """Filter blur """
        self.data = self.runcommand(self.convert,'-blur 0.5').getvalue()

    def filter_find_edges(self):
        """Filter contour """
        self.data = self.runcommand(self.convert,'-edge 0.5').getvalue()

    def filter_emboss(self):
        """Filter emboss """
        self.data = self.runcommand(self.convert,'-emboss 0.5').getvalue()

    def filter_smooth(self):
        """Filter smooth """
        self.data = self.runcommand(self.convert,'+noise Uniform').getvalue()

    def filter_sharpen(self):
        """Filter sharpen """
        self.data = self.runcommand(self.convert,'-sharpen 0.5').getvalue()

    def get_info(self):
        # we use identify to get information about the photo
        try: # maybe dont find identify :(
            res = self.runcommand(self.identify,'-verbose',binary=0)
            res = string.split(res,'\n')
            dict = {}
            for elem in res:
                list = string.split(elem,':')
                if(len(list)==2):
                    list[0] = string.strip(list[0])
                    list[1] = string.strip(list[1])
                    dict[list[0]] = list[1]
            del dict['Image']
            if(dict.has_key('Format')):
                dict['format'] = dict['Format']
            else:
                dict['format'] = 'Unknown'
            
            if(dict.has_key('Colorspace')):
                dict['mode'] = dict['Colorspace']
            else:
                dict['mode'] = 'Unknown'
            dict['width'] =  string.strip(string.split(dict['Geometry'],'x')[0])
            dict['height'] = string.strip(string.split(dict['Geometry'],'x')[1])
            return dict
        except:
            print traceback.print_exc()
            return {}

    def get_size(self):
        try: # maybe dont find identify :(
            res = self.runcommand(self.identify,'-format %wx%h',binary=0)
        except:
            return ''
        width,height = string.split(res,'x')
        height = string.strip(height)
        return int(width),int(height)

    def getrotate(self,angle):
        """rotate a photo with a given angle"""
        angle = int(angle)
        self.data = self.runcommand(self.convert,'-rotate %i' % angle).getvalue()

    def invert_image(self):
        self.data = self.runcommand(self.convert,'-negate').getvalue()

    def equalize(self):
        self.data = self.runcommand(self.convert,'-equalize').getvalue()

    def transpose_left_right(self):
        self.data = self.runcommand(self.convert,'-flop').getvalue()

    def sharpen(self):
        self.data = self.runcommand(self.convert,'-sharpen 1x0.5').getvalue()

    def solarize(self,thresholt): # thresholt is 0 to 99,9%
        self.data = self.runcommand(self.convert,'-solarize %i' % int(thresholt)).getvalue()
    
    def change_Brightness(self,brightness):
        self.data = self.runcommand(self.convert,'-modulate %ix100x100' % int(brightness)).getvalue()

    def change_constrast(self,contrast):
        self.data = self.runcommand(self.convert,'-contrast %i' % int(contrast)).getvalue()

    def export(self,format,width,height): # we dont write on image... we want a new one!
        olddata = self.data
        self.resize_image(height,width)
        res = self.runcommand(self.convert,'',1,string.lower(format))
        self.data = olddata # get oldinfo
        return res

    def resize_image(self,height,width):
        assert height!=0.0,  'Height is 0! Impossible!'
        assert width!=0.0, 'Width is 0! Impossible!'
        size = "%ix%i" % (width,height)
        self.data = self.runcommand(self.convert,'-size %s' % size).getvalue()

    def hex_to_rbg(self,string):
        a = int(string[1],16)
        b = int(string[2],16)
        c = int(string[3],16)
        d = int(string[4],16)
        e = int(string[5],16)
        f = int(string[6],16)

        x = (a * 16) + b;
        y = (c * 16) + d;
        z = (e * 16) + f;
        tuple = (x,y,z)
        return tuple
    
    def gettextcopyright(self, textcopy, textpos, textalign, textfont, textsize, textboldfont=0, textitalicfont=0, textcolor='#FF0000'):
        """copyright a photo with a given text

            boldfont:   0 ... normal font (the default)
                        1 ... bold font

            textpos:    The position of the copyright text can be specified by a string
                        containing a combination of one characters.  One character
                        must be either t, b or m.

            textalign:  The Alignemnt of the copyright text can be specified by a string
                        containing a combination of one characters.  One character
                        must be either l, c or r.

                    t ... top
                    b ... bottom
                    m ... middle

                    l ... left
                    c ... center
                    r ... right

                    The default value is 'bc'

        """
        width,height = self.get_size()
        drawList = []
        if textboldfont == '1':
            if textitalicfont == '1':
                font = '-font garamond -style Italic -stretch ExtraExpanded '
            else:
                font = '-font garamond -style Normal -stretch ExtraExpanded '

        elif textitalicfont == '1':
            font = '-font garamond -style Italic -stretch Normal '
        else:
            font = '-font garamond -style Normal -stretch Normal '
        tw = 30
        th = 12
#        tw -= 2
#        th -= 2
        if 't' in textpos:
            y = th
        elif 'm' in textpos:
            y = (height/2) - th - 2
        else: # The default value is 'b'
            y = height - th - 2
        if 'l' in textalign:
            x = 2
        elif 'r' in textalign:
            x = (width - tw) - 2
        else: # The default value is 'c'
            x = (width - tw) / 2
       # draw = ImageDraw.Draw(img)
        textcol = self.hex_to_rbg(textcolor)
        color = '-fill "rgb(%i,%i,%i)" ' % (textcol[0],textcol[1],textcol[2])
        draw = """-draw 'text %i,%i "%s"' """ % (x,y,textcopy)
        size = '-pointsize %i ' % int(textsize)
        #print font + color + draw
        self.data = self.runcommand(self.convert,font + color + size + draw).getvalue()

    def runcommand(self,command,arg,binary=1,outformat=''):
        if(outformat !=''):
            outformat = outformat + ':'
        command = '%s %s - %s-' % (command,arg,outformat)
        print 'running %s' % command
        if sys.platform == 'win32':
                    from win32pipe import popen2
                    stdin, stdout = popen2(command, 'b')
        else:
                    from popen2 import Popen3
                    cmd=Popen3(command)
                    stdout=cmd.fromchild
                    stdin=cmd.tochild
        print 'writing %i' % len(self.data)
        stdin.write(self.data)
        stdin.close()
        if(binary):
            res = StringIO()
            res.write(stdout.read())
            stdout.close()
        else:
            res = stdout.read()
            stdout.close()
        return res

    def remove_color(self):
        self.data = self.runcommand(self.convert,'-type Grayscale').getvalue()

    def add_borders(self,border=0,fill='#FFFFFF'):
        self.data = self.runcommand(self.convert,"-mattecolor '%s' -frame %ix%i" % (fill,int(border),int(border))).getvalue()

    def crop_image(self,border):
        width,height = self.get_size()
        self.data = self.runcommand(self.convert,"-crop %ix%i" % (width - int(border),height - int(border))).getvalue()

    def save(self):
        return StringIO(self.data)

    def modify_from_request(self,REQUEST):
        modif = 0
        if(hasattr(REQUEST,'copyright') and REQUEST.copyright=='on'):
                self.gettextcopyright(REQUEST.textcopy, REQUEST.position, REQUEST.align, REQUEST.textfont, REQUEST.textsize, REQUEST.textboldfont, REQUEST.textitalicfont, REQUEST.textcolor)
                modif = 1
        if(hasattr(REQUEST,'rotate') and REQUEST.rotate=='on'):
                self.getrotate(REQUEST.angle)
                modif = 1
        if(hasattr(REQUEST,'grayscale') and REQUEST.grayscale=='on'):
                self.remove_color()
                modif = 1
        if(hasattr(REQUEST,'change_size') and REQUEST.change_size=='on'):
                width = float(REQUEST.width)
                height = float(REQUEST.height)
                if(hasattr(REQUEST,'keep_ratio') and REQUEST.keep_ratio=='on'):
                        oldheight, oldwidth = self.get_size()
                        oldheight=float(oldheight)
                        oldwidth=float(oldwidth)
                        print "%i %i %i %i" % (width,height,oldwidth,oldheight)
                        ratio = oldwidth/oldheight
                        if(oldheight!=height and oldwidth==width): # height is modified
                                width = ratio * height
                        elif(oldheight==height and oldwidth!=width): # width is modified
                                height = width/ratio
                        elif(oldheight!=height and oldwidth!=width):
                                width = ratio * height
                        self.resize_image(int(height),int(width))
                else:
                        self.resize_image(height,width)
                modif = 1

        if(hasattr(REQUEST,'change_brightness') and REQUEST.change_brightness=='on'):
                modif = 1
                self.change_Brightness(int(REQUEST.brightness) % 100)

        if(hasattr(REQUEST,'change_contrast') and REQUEST.change_contrast=='on'):
                modif = 1
                self.change_contrast(int(REQUEST.contrast) % 100)

        if(hasattr(REQUEST,'filter_blur') and REQUEST.filter_blur=='on'):
                modif = 1
                self.filter_blur()

        if(hasattr(REQUEST,'filter_emboss') and REQUEST.filter_emboss=='on'):
                modif = 1
                self.filter_emboss()

        if(hasattr(REQUEST,'filter_find_edges') and REQUEST.filter_find_edges=='on'):
                modif = 1
                self.filter_find_edges()

        if(hasattr(REQUEST,'filter_smooth') and REQUEST.filter_smooth=='on'):
                modif = 1
                self.filter_smooth()

        if(hasattr(REQUEST,'filter_sharpen') and REQUEST.filter_sharpen=='on'):
                modif = 1
                self.filter_sharpen()

        if(hasattr(REQUEST,'equalize') and REQUEST.equalize=='on'):
                modif = 1
                self.equalize()

        if(hasattr(REQUEST,'add_border') and REQUEST.add_border=='on'):
                modif = 1
                borderadd = int(REQUEST.border_to_add)
                fill_border = REQUEST.fill
                self.add_borders(borderadd,fill_border)

        if(hasattr(REQUEST,'crop_image') and REQUEST.crop_image=='on'):
                modif = 1
                border = int(REQUEST.remove_border)
                self.crop_image(border)

        if(hasattr(REQUEST,'invert_image') and REQUEST.invert_image=='on'):
                modif = 1
                self.invert_image()

        if(hasattr(REQUEST,'transpose_left_right') and REQUEST.transpose_left_right=='on'):
                modif = 1
                self.transpose_left_right()

        if(hasattr(REQUEST,'solarize') and REQUEST.solarize=='on'):
                modif = 1
                self.solarize(REQUEST.threshold)
        return (self.save(),modif)

if __name__ == "__main__":
        fd = open('/var/www/photos/screencapture/edit_photo.jpg')
        data = fd.read()
        fd.close()
        a = IM_Image_modif('/usr/bin/convert',data)
        print a.get_info()
        #res = a.getrotate(90)
        #res = a.invert_image()
        #res = a.equalize()
        #res = a.transpose_left_right()
        #res = a.solarize(40)
        #res = a.change_Brightness(40)
        #print a.get_size()
        #res = a.resize_image(200,200)
        #a.gettextcopyright("supercooleraezre", ['b'],['l'],"", 20, textboldfont='1', textitalicfont='1',textcolor='#1AAEEE')
        #a.add_borders(20,'#1AAEEE')
        #a.crop_image(40)
        #a.remove_color()
        res = a.save()
        fd = open('/tmp/result','wb')
        fd.write(res.getvalue())
        fd.close()
        
